@@ -4,3 +4,6 @@ CHECKPOINT_FILENAME_TEMPLATE = "{}.{}.pth"
 BEST_IDENTIFIER = "best"
 BEST_CHECKPOINT_FILENAME_TEMPLATE = "{}.best.pth"
 LOG_FILENAME = "log.log"
+
+from .relation_extraction import MCMLSentRelationClassificationTask
+from .simple_task import SimpleTask
