@@ -2,12 +2,12 @@ import os
 import random
 from typing import Optional
 
-import torch
 import numpy as np
+import torch
 from omegaconf.omegaconf import OmegaConf
 
+from rex.tasks import CONFIG_PARAMS_FILENAME, LOG_FILENAME
 from rex.utils.logging import logger
-from rex.tasks import LOG_FILENAME, CONFIG_PARAMS_FILENAME
 
 
 def set_seed(seed: Optional[int] = 1227, set_cudnn: Optional[bool] = False):
@@ -44,6 +44,7 @@ def init_all(
 ):
     set_seed(seed, set_cudnn=set_cudnn)
     logger.add(os.path.join(task_dir, LOG_FILENAME), backtrace=True, diagnose=True)
+
     if config is not None:
         OmegaConf.save(
             config, os.path.join(task_dir, CONFIG_PARAMS_FILENAME), resolve=True
