@@ -1,6 +1,9 @@
 import json
 import pickle
-from typing import Iterable, Any, Optional, List
+from typing import Any, Iterable, List, Optional
+
+from rex.utils.deprecation import deprecation_warning
+from rex.utils.logging import logger
 
 
 def dump_json(obj, filepath, **kwargs):
@@ -16,6 +19,11 @@ def load_json(filepath, **kwargs):
 
 
 def dump_line_json(obj, filepath, **kwargs):
+    deprecation_warning("dump_line_json", "dump_jsonlines")
+    return dump_jsonlines(obj, filepath, **kwargs)
+
+
+def dump_jsonlines(obj, filepath, **kwargs):
     with open(filepath, "wt", encoding="utf-8") as fout:
         for d in obj:
             line_d = json.dumps(d, ensure_ascii=False, **kwargs)
@@ -23,6 +31,11 @@ def dump_line_json(obj, filepath, **kwargs):
 
 
 def load_line_json(filepath, **kwargs):
+    deprecation_warning("load_line_json", "load_jsonlines")
+    return load_jsonlines(filepath, **kwargs)
+
+
+def load_jsonlines(filepath, **kwargs):
     data = list()
     with open(filepath, "rt", encoding="utf-8") as fin:
         for line in fin:
@@ -37,7 +50,7 @@ def dump_pickle(obj, filepath, **kwargs):
 
 
 def load_pickle(filepath, **kwargs):
-    data = []
+    data = None
     with open(filepath, "rb") as fin:
         data = pickle.load(fin, **kwargs)
     return data
@@ -90,15 +103,17 @@ def load_csv(
     return data
 
 
-def load_embedding_file(filepath, encoding="utf-8"):
+def load_embedding_file(filepath, encoding="utf-8", open_func=open, verbose=False):
     tokens = []
     token2vec = {}
+    num_tokens = -1
     dim_emb = 0
-    with open(filepath, "rt", encoding=encoding) as fin:
+    with open_func(filepath, "rt", encoding=encoding) as fin:
         for line_no, line in enumerate(fin):
             line = line.split()
             if line_no == 0:
                 if len(line) == 2 and all(x.isdigit() for x in line):
+                    num_tokens = int(line[0])
                     dim_emb = int(line[1])
                 else:
                     dim_emb = len(line) - 1
@@ -110,6 +125,15 @@ def load_embedding_file(filepath, encoding="utf-8"):
                 continue
             tokens.append(line[0])
             token2vec[line[0]] = list(map(float, line[1:]))
+
+    if num_tokens > 0 and num_tokens != len(tokens):
+        logger.warning(
+            f"emb file info num of tokens: {num_tokens}, while {len(tokens)} tokens are found"
+        )
+
+    if verbose:
+        logger.info(f"Loading #Tokens: {len(tokens)}, Emb dim: {dim_emb}")
+
     return tokens, token2vec
 
 
@@ -119,7 +143,12 @@ def load_line_iterator(filepath):
             yield line
 
 
-def load_line_json_iterator(filepath):
+def load_line_json_iterator(filepath, **kwargs):
+    deprecation_warning("load_line_json_iterator", "load_jsonlines_iterator")
+    return load_jsonlines_iterator(filepath, **kwargs)
+
+
+def load_jsonlines_iterator(filepath):
     for line in load_line_iterator(filepath):
         yield json.loads(line)
 
